@@ -124,18 +124,28 @@ enchant_unlock_file (FILE * f)
 static char *
 enchant_get_module_dir (void)
 {
+	char * module_dir = NULL;
+
 #ifdef XP_TARGET_COCOA
 	return g_strdup ([[EnchantResourceProvider instance] moduleFolder]);
 #endif
-	char * module_dir = NULL;
 
+	/* Look for explicitly set registry values */
 	module_dir = enchant_get_registry_value ("Config", "Module_Dir");
 	if (module_dir)
 		return module_dir;
 
-#ifdef ENABLE_BINRELOC
-	return g_strdup (BR_LIBDIR ("/enchant"));
-#elif defined(ENCHANT_GLOBAL_MODULE_DIR)
+	/* Dynamically locate library and search for modules relative to it. */
+	char * prefix = NULL;
+	prefix = enchant_get_prefix_dir();
+	if(prefix)
+		{
+			module_dir = g_build_filename(prefix,"lib","enchant",NULL);
+			g_free(prefix);
+			return module_dir;
+		}
+
+#if defined(ENCHANT_GLOBAL_MODULE_DIR)
 	return g_strdup (ENCHANT_GLOBAL_MODULE_DIR);
 #else
 	return NULL;
@@ -148,15 +158,23 @@ enchant_get_conf_dir (void)
 #ifdef XP_TARGET_COCOA
 	return g_strdup ([[EnchantResourceProvider instance] configFolder]);
 #endif
-	char * ordering_dir = NULL;
+	char * ordering_dir = NULL, * prefix = NULL;;
 
+	/* Look for explicitly set registry values */
 	ordering_dir = enchant_get_registry_value ("Config", "Data_Dir");
 	if (ordering_dir)
 		return ordering_dir;
 
-#ifdef ENABLE_BINRELOC
-	return g_strdup (BR_DATADIR ("/enchant"));
-#elif defined(ENCHANT_GLOBAL_ORDERING)
+	/* Dynamically locate library and search for files relative to it. */
+	prefix = enchant_get_prefix_dir();
+	if(prefix)
+		{
+			ordering_dir = g_build_filename(prefix,"share","enchant",NULL);
+			g_free(prefix);
+			return ordering_dir;
+		}
+
+#if defined(ENCHANT_GLOBAL_ORDERING)
 	return g_strdup (ENCHANT_GLOBAL_ORDERING);
 #else
 	return NULL;
@@ -1261,8 +1279,8 @@ enchant_broker_list_dicts (EnchantBroker * broker,
 					char ** dicts;				       
 
 					dicts = (*provider->list_dicts) (provider, &n_dicts);
-					name = (*provider->identify) (provider);
-					desc = (*provider->describe) (provider);
+					name = (provider->identify ? (*provider->identify) (provider) : "");
+					desc = (provider->describe ? (*provider->describe) (provider) : "");
 					file = g_module_name (module);
 
 					for (i = 0; i < n_dicts; i++)
@@ -1307,6 +1325,51 @@ enchant_broker_free_dict (EnchantBroker * broker, EnchantDict * dict)
 		g_hash_table_remove (broker->dict_map, session->personal_filename);
 }
 
+static int
+_enchant_provider_dictionary_exists (EnchantProvider * provider,
+				     const char * const tag)
+{
+	int exists = 0;
+
+	if (provider->dictionary_exists)
+		{
+			exists = (*provider->dictionary_exists) (provider, tag);
+		}
+	else if (provider->list_dicts)
+		{
+			const char * tag, * name, * desc, * file;
+			size_t n_dicts, i;
+			char ** dicts;				       
+			
+			dicts = (*provider->list_dicts) (provider, &n_dicts);
+			
+			for (i = 0; i < n_dicts; i++)
+				{
+					if (!strcmp(dicts[i], tag)) 
+						{
+							exists = 1;
+							break;
+						}
+				}
+			
+			enchant_provider_free_string_list (provider, dicts);
+		}
+	else if (provider->request_dict)
+		{
+			EnchantDict *dict = NULL;
+
+			dict = (*provider->request_dict) (provider, tag);
+			if (dict)
+				{
+					if (provider->dispose) 
+						(*provider->dispose) (provider);
+					exists = 1;
+				}
+		}
+
+	return exists;
+}
+
 /**
  * enchant_broker_dict_exists
  * @broker: A non-null #EnchantBroker
@@ -1339,12 +1402,10 @@ enchant_broker_dict_exists (EnchantBroker * broker,
 		{
 			provider = (EnchantProvider *) list->data;
 
-			if (provider->dictionary_exists)
+			if (_enchant_provider_dictionary_exists (provider, normalized_tag))
 				{
-					if ((*provider->dictionary_exists) (provider, normalized_tag)) {
-						g_free (normalized_tag);
-						return 1;
-					}
+					g_free (normalized_tag);
+					return 1;
 				}
 		}
 
@@ -1438,7 +1499,7 @@ enchant_broker_get_error (EnchantBroker * broker)
 
 /* private */ 
 ENCHANT_MODULE_EXPORT(char *)
-_enchant_get_user_language(void)
+enchant_get_user_language(void)
 {
 	char * locale = NULL;
 	
@@ -1465,3 +1526,39 @@ _enchant_get_user_language(void)
 		
 	return locale;
 }
+
+
+/**
+ * enchant_get_prefix_dir
+ *
+ * Returns a string giving the location of the base directory
+ * of the enchant installation.  This corresponds roughly to 
+ * the --prefix option given to ./configure when enchant is
+ * compiled, except it is determined at runtime based on the location
+ * of the enchant library.
+ *
+ * This API is private to the providers.
+ *
+ */
+ENCHANT_MODULE_EXPORT (char *)
+enchant_get_prefix_dir(void)
+{
+	char * prefix = NULL;
+
+#ifdef _WIN32
+	/* Dynamically locate library and return containing directory */
+	HINSTANCE hInstance = GetModuleHandle("libenchant-1");
+	if(hInstance != NULL)
+		{
+			char dll_path[MAX_PATH];
+
+			if(GetModuleFileName(hInstance,dll_path,MAX_PATH))
+				prefix = g_path_get_dirname(dll_path);
+		}
+#elif defined(ENABLE_BINRELOC)
+	/* Use standard binreloc PREFIX macro */
+	prefix = g_strdup (PREFIX);
+#endif
+
+	return prefix;
+}
